/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtMultimedia 5.0
import Qt.labs.settings 1.0
import Ubuntu.Components 1.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.PushNotifications 0.1
import U1db 1.0 as U1db
import Telegram 0.1
import "js/version.js" as Version
import "ui"
import "components"

MainView {
    property bool initCalled: false
    property bool connected: false

    property alias telegramClient: telegramClientLoader.item
    property alias pushClient: pushClientLoader.item
    property alias pushDialog: pushDialogLoader.item

    property string appId: "com.ubuntu.telegram_telegram"
    property bool promptForPush:
            (typeof appSettings.contents.promptForPush === "undefined")
            ? false: appSettings.contents.promptForPush

    property var uri: undefined
    property int chatToOpen: -1
    property var chat: undefined

    id: root
    // For autopilot-qt4 functional testing.
    objectName: "mainView"

    // Must match "name" in click manifest.
    applicationName: "com.ubuntu.telegram"

    automaticOrientation: true

    useDeprecatedToolbar: false

    anchorToKeyboard: true

    width: units.gu(45)
    height: units.gu(80)

    onActiveFocusChanged: {
        if (initCalled) {
            if (activeFocus) {
                console.log("Telegram resuming..");
                telegramClient.wake();

                openPendingChats();
            } else {
                console.log("Telegram suspending..");
                telegramClient.sleep();
            }
        }
        if (activeFocus) {
            if (pushClientLoader.status === Loader.Ready) {
                // We do this to register for push when U1 account was
                // set-up after Telegram was run without it first.
                pushClient.appId = root.appId;
            }
        }
    }

    Connections {
        target: UriHandler
        onOpened: {
            root.uri = uris[0];
            processUri();
        }
    }

    QtObject {
        id: userData
        property string phoneNumber: ""
        property string firstName: ""
        property string lastName: ""

        function reset() {
            phoneNumber = "";
            firstName = "";
            lastName = "";
        }
    }

    PageStack {
        id: pageStack

        // --- intro page
        PageLoader {
            id: introPageLoader
            sourceComponent: IntroPage {
                id: introPage
            }
            onLoaded: pageStack.showIntroPage()
        }

        Connections {
            ignoreUnknownSignals: true
            target: introPageLoader.valid ? introPageLoader.item : null
            onShowSignInPage: pageStack.showSignInPage()
        }

        function showIntroPage() {
            if (introPageLoader.status === Loader.Ready) {
                pageStack.clear();
                pageStack.push(introPageLoader.item);
            } else {
                introPageLoader.active = true;
            }
        }
        // --- end of: intro page

        function showSignInPage() {
            pageStack.push(signInPage);
            appSettings.set("isFirstLaunch", false);
        }

        Component {
            id: signInPage
            SignInPage {}
        }

        Component {
            id: picker
            PickerPage {}
        }

        Component {
            id: signUpPage
            SignUpPage {}
        }

        Component {
            id: verificationCodePage
            VerificationCodePage {
                objectName: "verificationPage"
            }
        }

        Component {
            id: dialogsPage
            DialogsPage {}
        }

        Component {
            id: dialogPage
            DialogPage {
                objectName: "chatPage"
            }
        }

        Component {
            id: profilePage
            ProfilePage {}
        }

        Component {
            id: groupProfilePage
            GroupProfilePage {}
        }

        Component {
            id: contactsPage
            ContactsPage {}
        }

        Component {
            id: addContactPage
            AddContactPage {}
        }

        Component {
            id: settingsPage
            SettingsPage {}
        }

        Component {
            id: previewPage
            PreviewPage {}
        }

        function onFirstLaunch() {
            showIntroPage();
        }

        function onLaunch() {
            pageStack.clear();
            pageStack.push(signInPage);
        }

        function onLoggedInLaunch() {
            pageStack.clear();
            pageStack.push(dialogsPage);

            var phone = appSettings.contents.phoneNumber;
            console.log("reusing phone number: " + phone);
            setupTelegram(phone);
        }
    }

    function setupTelegram(phoneNumber) {
        telegramClient.phoneNumber = phoneNumber;
        telegramClient.init();
        initCalled = true;
    }

    function openChat(title, peerId, peerType) {
        if (pageStack.currentPage.objectName === "chatPage") {
            // Close any current conversation page first.
            pageStack.pop();
        }

        var properties = {title: title, peerId: peerId, peerType: peerType};
        pageStack.push(dialogPage, properties);
    }

    function newChat() {
        openContacts(false);
    }

    function newGroupChat() {
        openContacts(true);
    }

    function openProfile(userId) {
        var properties = {userId: userId};
        pageStack.push(profilePage, properties)
    }

    function openGroupProfile(chatId) {
        var properties = {chatId: chatId};
        pageStack.push(groupProfilePage, properties)
    }

    function openContacts(groupChatMode) {
        var properties = { groupChatMode: groupChatMode };
        pageStack.push(contactsPage, properties);
    }

    function openSettings() {
        pageStack.push(settingsPage);
    }

    function showAddContactPage(addManually, peerId, contact) {
        var properties = { addManually: addManually, peerId: peerId };
        if (contact !== undefined) {
            properties["contact"] = contact;
        }
        pageStack.push(addContactPage, properties);
    }

    function doUpgradeToDatabase() {
        if (appSettings.contents.loadFromSettings) {
            console.log("Upgrading from Settings to U1db.Database")
            appSettings.set("isFirstLaunch", appSettingsLegacy.isFirstLaunch);
            appSettings.set("isLoggedIn", appSettingsLegacy.isLoggedIn);
            appSettings.set("phoneNumber", appSettingsLegacy.phoneNumber);
            appSettings.set("promptForPush", appSettingsLegacy.promptForPush);
            appSettings.set("loadFromSettings", false);
        }
    }

    Settings {
        // TODO Update Qt.application.domain to fix this path:
        // ~/.config/qt-project.org/com.ubuntu.telegram.conf
        id: appSettingsLegacy
        category: "session"

        property bool isFirstLaunch: true
        property bool isLoggedIn: false
        property string phoneNumber: ""
        property bool promptForPush: true
    }

    U1db.Database {
        id: database
        // TODO: change /home/phablet to ~ as soon as bug #1387294 is fixed
        path: "/home/phablet/.config/%1/settings.u1db".arg(applicationName)
    }

    U1db.Document {
        id: appSettings
        database: database
        docId: "appSettings"
        defaults: {
            "loadFromSettings": true,
            "isFirstLaunch": true,
            "isLoggedIn": false,
            "phoneNumber": "",
            "promptForPush": true
        }
        create: true

        function set(key, value) {
            var tmp = contents;
            tmp[key] = value;
            contents = tmp;

            if (key === "phoneNumber") {
                // remove spaces and +
                userSettings.setNumber(value);
            }
        }

        Component.onCompleted: {
            // WORKAROUND: it seems that when the object is created, the contents aren't
            // still set. We force the update of its contents
            if (database.listDocs().indexOf(docId) < 0) {
                contents = defaults;
            }
            doUpgradeToDatabase();
        }

    }

    U1db.Document {
        id: userSettings
        database: database
        create: false
        defaults: { "sendByEnter": false, "chatBackground": "", "fontSize": "medium" }

        function set(key, value) {
            var tmp = contents;
            tmp[key] = value;
            contents = tmp;
        }

        function setNumber(phoneNumber) {
            docId = Qt.md5(phoneNumber.replace(/[\s\+\(\)]/g, ''));
            if (database.listDocs().indexOf(docId) < 0) {
                contents = defaults;
            }
        }

        Component.onCompleted: {
            if (typeof appSettings.contents.phoneNumber != "undefined") {
                setNumber(appSettings.contents.phoneNumber);
            }
        }
    }

    Loader {
        id: pushClientLoader
        active: false
        asynchronous: false
        sourceComponent: PushClient {
            id: pushClient
            // TODO karni on 2014-11-27: REMOVE this next line when push-client fix lands.
            appId: root.appId

            property bool connected: telegramClient.connected
            property bool registered: false

            signal tokenChanged(string token);

            onTokenChanged: {
                console.log("push: token changed -- " + (token.length > 0 ? "present" : "empty!"));
                if (token.length > 0) {
                    registered = false;
                    registerForPush();
                }
            }

            onError: {
                if (status == "bad auth") {
                    console.warn("push - 'bad auth' error: " + status);
                    if (promptForPush) {
                        pushDialogLoader.active = true;
                    }
                } else {
                    console.warn("push - unhandled error: " + status);
                }
            }

            onConnectedChanged: {
                if (token.length > 0) {
                    registerForPush();
                }
            }

            function registerForPush() {
                if (registered) return;

                if (token.length === 0) {
                    console.warn("push - can't register, empty token.");
                    return;
                }
                if (!connected) {
                    console.warn("push - can't register, not connected yet.");
                    return;
                }
                if (!appSettings.contents.isLoggedIn) {
                    console.warn("push - can't register, not logged-in yet.");
                    return;
                }

                telegramClient.accountRegisterDevice(token, Version.version);
            }

            function unregisterFromPush() {
                if (registered) {
                    telegramClient.accountUnregisterDevice(token);
                } else {
                    console.warn("push: Could not unregister from push!");
                }
            }
        }

        onStatusChanged: {
            if (status === Loader.Loading) {
                console.debug("push - push client loading");
            } else if (status === Loader.Ready) {
                console.debug("push - push client loaded");
            }
        }

        onLoaded: {
            pushClient.notificationsChanged.connect(telegramClient.onPushNotifications);
            pushClient.error.connect(telegramClient.onPushError);

            if (!appSettings.contents.isLoggedIn) {
                // This should never happen, as we start the Loader in onAuthLoggedIn.
                console.error("push - loaded pushClient, but not logged in!");
                return;
            }

            // This should trigger onTokenChanged, and cause a call to getNotifications()
            // TODO karni on 2014-11-27: UNCOMMENT this next line when push-client fix lands.
            //pushClient.appId = root.appId;
            // TODO karni on 2014-11-27: REMOVE this next line when push-client fix lands.
            pushClient.registerForPush();
        }
    }

    Loader {
        id: telegramClientLoader
        active: false
        asynchronous: false
        sourceComponent: TelegramClient {
            id: telegramClient
            configPath: "~/.config/" + applicationName
            publicKeyFile: "telegram.pub"
            property real lastCheckedId: 0

            function logoutResetSession(showIntro) {
                appSettings.set("isLoggedIn", false);
                appSettings.set("phoneNumber", "");
                appSettings.set("isFirstLaunch", showIntro);
                initCalled = false;
            }

            function logoutResetUI(showIntro) {
                pageStack.clear();
                if (showIntro) {
                    pageStack.showIntroPage();
                } else {
                    pageStack.push(signInPage);
                }
            }

            function logout(showIntro) {
                if (typeof showIntro === "undefined") {
                    showIntro = false;
                }

                logoutResetSession(showIntro);
                logoutResetUI(showIntro);
            }

            onError: {
                var msg = errorCode + " " + errorText;
                if (errorCode < 500) {
                    console.warn("Client error: " + msg);
                } else {
                    console.warn("Server error: " + msg);
                }

                if (errorCode === 401) {
                    if (errorText === "SESSION_REVOKED") {
                        telegramClient.logout(true);
                    } else {
                        // We want to reset session now. When app is re-launched
                        // manually, it should not be signed in already.
                        telegramClient.logoutResetSession();
                        openLoggedOutDialog();
                    }
                }
            }

            onFatalError: {
                logout();
            }

            onAuthNeeded: {
                authCheckPhone();
            }

            onAuthCheckPhoneSent: {
                if (telegramClient.phoneNumber === phoneNumber) {
                    telegramClient.lastCheckedId = msgId;
                }
            }

            onAuthCheckPhoneAnswer: {
                if (id !== lastCheckedId) return;
                if (phoneRegistered) {
                    authSendCode();
                } else {
                    pageStack.push(signUpPage);
                }
            }

            onAuthSendCallAnswer: {
                verificationCodePage.calling();
            }

            onAuthLoggedIn: {
                if (!appSettings.contents.isLoggedIn) {
                    appSettings.set("isLoggedIn", true);
                    appSettings.set("phoneNumber", userData.phoneNumber);
                    appSettings.set("isFirstLaunch", false);
                }

                if (pageStack.currentPage.objectName === "verificationPage") {
                    pageStack.clear();
                    pageStack.push(dialogsPage);
                }

                dialogsModel.setTelegramClient(telegramClient);

                if (!pushClientLoader.active) {
                    pushClientLoader.active = true;
                } else if (pushClientLoader.status === Loader.Ready) {
                    pushClient.registerForPush();
                }

                processUri();
                openPendingChats();
            }

            onAuthLogOutAnswer: {
                logout(!ok);
            }

            onAuthResetAuthorizationsAnswer: {
                console.log("terminated all other sessions");
                if (ok) {
                    console.log("registering again for push notifications");
                    pushClient.registered = false;
                    pushClient.registerForPush();
                }
            }

            onAccountRegisterDeviceAnswer: {
                if (ok) {
                    console.log("push - registered with Telegram");
                } else {
                    console.warn("push - failed to register with Telegram!");
                }
                pushClient.registered = ok;
            }

            onAccountUnregisterDeviceAnswer: {
                if (ok) {
                    console.log("push - unregistered from Telegram");
                } else {
                    console.warn("push - failed to unregister from Telegram!");
                }
                // Set unregistered state regardless if it succeeded.
                pushClient.registered = false;
            }

            onWoken: {
                if (pushClient.registered) {
                    pushClient.getNotifications();
                }
            }

            onMessagesCreateChatAnswer: {
                openChat(chat.title, chat.id, TLInputPeer.TypeInputPeerChat);
            }
        }

        onLoaded: {
            if (appSettings.contents.isFirstLaunch) {
                pageStack.onFirstLaunch();
            } else if (!appSettings.contents.isLoggedIn) {
                pageStack.onLaunch();
            } else {
                pageStack.onLoggedInLaunch();
            }
        }
    }

    DialogsModel {
        id: dialogsModel
    }

    function openPushDialog() {
        // karni: Consider moving this to userSettings instead.
        appSettings.set("promptForPush", true);
        if (pushDialogLoader.status === Loader.Ready) {
            PopupUtils.open(pushDialogComponent);
        } else {
            pushDialogLoader.active = true;
        }
    }

    Loader {
        id: pushDialogLoader
        active: false
        asynchronous: false
        sourceComponent: Component {
            id: pushDialogComponent
            Dialog {
                id: pushDialog
                title: i18n.tr("Push notifications")
                text: i18n.tr("To enable push, sign in to your Ubuntu One account.")
                Button {
                    text: i18n.tr("Sign in to Ubuntu One")
                    color: UbuntuColors.orange
                    onClicked: {
                        Qt.openUrlExternally("settings:///system/online-accounts");
                        close();
                    }
                }
                Button {
                    text: i18n.tr("Remind me later")
                    onClicked: close()
                }
                Button {
                    text: i18n.tr("Don't use push")
                    onClicked: {
                        appSettings.set("promptForPush", false);
                        pushClient.unregisterFromPush();
                        close();
                    }
                }

                function close() {
                    PopupUtils.close(pushDialog);
                }
            }
        }

        onLoaded: {
            openPushDialog();
        }
    }

    function openLoggedOutDialog() {
        if (loggedOutDialogLoader.status === Loader.Ready) {
            PopupUtils.open(loggedOutDialogComponent);
        } else {
            loggedOutDialogLoader.active = true;
        }
    }

    Loader {
        id: loggedOutDialogLoader
        active: false
        asynchronous: false
        sourceComponent: Component {
            id: loggedOutDialogComponent
            Dialog {
                id: loggedOutDialogDialog
                title: i18n.tr("Unauthorized")
                text: i18n.tr("You have been logged out. Please log in.")
                Button {
                    text: i18n.tr("OK")
                    color: UbuntuColors.orange
                    onClicked: {
                        telegramClient.logout();
                        close();
                    }
                }

                function close() {
                    PopupUtils.close(loggedOutDialogDialog);
                }
            }
        }

        onLoaded: {
            openLoggedOutDialog();
        }
    }

    Component.onCompleted: {
        if (typeof uriArgs !== 'undefined' && uriArgs) {
            root.uri = uriArgs[0];
            processUri();
        }

        telegramClientLoader.active = true;
    }

    function processUri() {
        if (root.uri === undefined) return;

        var commands = root.uri.split("://")[1].split("/");
        if (commands) {
            switch(commands[0].toLowerCase()) {
            case "chat":
                chatToOpen = parseInt(commands[1]);
                break;
            default: console.warn("Unmanaged URI! " + commands);
            }
        }
        commands = undefined;
    }

    Timer {
        // DialogsModel needs time to populate.
        id: pendingChatTimer
        interval: 500
        running: false
        repeat: false
        onTriggered: doOpenPendingChats()
    }

    function openPendingChats() {
        pendingChatTimer.restart();
    }

    function doOpenPendingChats() {
        if (chatToOpen === -1) return;
        chat = dialogsModel.getFromId(chatToOpen);
        console.log("opening: " + chat.name + ", " + chatToOpen + ", " + chat.peerType);
        openChat(chat.name, chatToOpen, chat.peerType);
        chatToOpen = -1;
    }
}
