import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import Ubuntu.Components.Popups 0.1
import Ubuntu.Contacts 0.1
import "../components"
import "../components/listitems"
import "../js/avatar.js" as Avatar
import "../js/time.js" as Time
import Telegram 0.1

TelegramPage {
    id: contactsPage

    // These values are passed on page creation.
    property bool groupChatMode: false
    property bool addToGroupMode: false
    property int groupChatId: -1
    property string groupChatTitle: ""

    property alias contactsModel: contactsModel
    property alias isSelectingGroup: contactListView.isInSelectionMode
    property bool isGroupCountSatisfied: contactListView.selectedItems.count > 1
    property var actionsNone: []
    property list<Action> actionsSearch: [
        Action {
            iconName: "search"
            text: i18n.tr("Search")
            onTriggered: searchPressed()
        }
    ]
    property list<Action> actionsNewChat: [
        Action {
            iconName: "search"
            text: i18n.tr("Search")
            onTriggered: searchPressed()
        },
        Action {
            iconName: "contact-new"
            text: i18n.tr("Add")
            enabled: isConnected
            onTriggered: addPressed()
        }
    ]
    property list<Action> actionsNewGroupChat: [
        Action {
            iconName: "ok"
            text: i18n.tr("Ok")
            enabled: isConnected && groupChatTitleTextField.length > 0
            onTriggered: createChatPressed()
        }
    ]

    title: {
        if (groupChatMode) {
            return i18n.tr("New group chat");
        } else if (addToGroupMode) {
            return i18n.tr("Add to group");
        } else {
            return i18n.tr("Contacts");
        }
    }

    head.actions: {
        if (isSearching) {
            return actionsNone;
        } else if (isSelectingGroup) {
            return isGroupCountSatisfied ? actionsNewGroupChat : actionsNone;
        } else {
            return actionsNewChat;
        }
    }

    body: Item {
        anchors.fill: parent

        Label {
            id: listEmptyLabel
            anchors.centerIn: parent
            fontSize: "medium"
            visible: contactListView.model.count === 0
            text: i18n.tr("No contacts")
        }

        ContactsModel {
            id: contactsModel
            Component.onCompleted: {
                contactsModel.setTelegramClient(telegramClient);
                contactsModel.updateQuery();
            }
        }

        ContactsProxy {
            id: contactsProxy
            sourceModel: contactsModel
            sortRole: ContactsModel.FirstNameRole
            ascending: true
            searchTerm: contactsPage.searchTerm
        }

        TextField {
            // Use height and opacity, we can't animate on visibility.
            property bool isVisible: contactListView.selectedItems.count > 1

            id: groupChatTitleTextField
            anchors {
                top: parent.top
                topMargin: units.gu(1)
                left: parent.left
                leftMargin: units.gu(1)
                right: parent.right
                rightMargin: units.gu(1)
            }
            height: isVisible ? units.gu(5) : 0
            opacity: isVisible ? 1.0 : 0.0
            placeholderText: i18n.tr("Chat title")

            Behavior on height {
                NumberAnimation { duration: 300 }
            }
            Behavior on opacity {
                NumberAnimation { duration: 300 }
            }
        }

        MultipleSelectionListView {
            id: contactListView
            anchors {
                top: groupChatTitleTextField.visible ? groupChatTitleTextField.bottom : parent.top
                topMargin: groupChatTitleTextField.isVisible ? units.gu(1) : 0
                left: parent.left
                right: parent.right
                bottom: parent.bottom
                bottomMargin: units.dp(4)
            }

            clip: true

            section {
                property: "firstName"
                criteria: ViewSection.FirstCharacter
                labelPositioning: ViewSection.InlineLabels
                delegate: ListItem.Header {
                    text: section != "" ? section : "#"
                }
            }

            listModel: contactsProxy
            listDelegate: TelegramContactsListItem {
                id: contactDelegate
                // no lastSeen, lp #1347491
                image: model.thumbnail !== "" ? model.thumbnail
                                              : Qt.resolvedUrl("../images/" + Avatar.getAvatar(model.id))
                title: model.firstName + (model.lastName.length > 0 ? " " + model.lastName : "")
                lastSeen: Time.formatLastSeen(model.lastSeenOnline * 1000)
                isOnline: model.online
                selected: contactListView.isSelected(contactDelegate)

                onClicked: {
                    if (contactListView.isInSelectionMode) {
                        if (!contactListView.selectItem(contactDelegate)) {
                            contactListView.deselectItem(contactDelegate);
                        } else if (contactListView.selectedItems.count >= 201) {
                            // TODO Find a cleaner way.
                            contactListView.deselectItem(contactDelegate);
                        }
                        contactListView.refreshSubtitle();
                    } else if (addToGroupMode) {
                        var dialog = PopupUtils.open(addContactToGroupChatComponent);
                        dialog.setData(
                                model.id, // user id
                                title,    // user name
                                groupChatTitle);
                    } else {
                        openSimpleChat(model);
                    }
                }
            }

            onSelectedItemsChanged: {
                refreshSubtitle();
            }

            onSelectionCanceled: {
                groupChatMode = false;
            }

            onSelectionDone: {
                createChat(items);

                groupChatMode = false;
                groupChatTitleTextField.text = "";
            }

            function refreshSubtitle() {
                if (groupChatMode) {
                    var count = contactListView.selectedItems.count;
                    var maxMembers = 200;
                    // TRANSLATORS: the first argument refers to the number of members
                    // on a group conversation. The second one refers to the maximum
                    // number of members that are supporte for groups (currently 200).
                    subtitle = i18n.tr("%1/%2 members").arg(count).arg(maxMembers);
                } else {
                    subtitle = "";
                }
            }
        }

        Scrollbar {
            flickableItem: contactListView
        }
    }

    ContactImport {
        id: contactImporter

        onContactsReceived: {
            busy = true;
            contactParser.vCardUrl = url;
        }
    }
    Component {
        id: contactImportDialogComponent
        Dialog {
            id: contactImportDialog
            title: contactParser.dialogTitle
            text: contactParser.dialogText
            Button {
                text: i18n.tr("OK")
                color: UbuntuColors.orange
                onClicked: PopupUtils.close(contactImportDialog)
            }
        }
    }

    VCardParser {
        id: contactParser

        signal addContactsAnswer(var id, var contactIds, var retryList, var urserList);

        property int importedContactCount: 0
        property string dialogTitle: ""
        property string dialogText: ""

        function parseContact(vcardContact) {
            var contact = Qt.createQmlObject(
                        'import Telegram 0.1; TLInputContact {}', contactParser);

            contact.firstName = vcardContact.name.firstName;
            contact.lastName = vcardContact.name.lastName;
            if (contact.firstName === "") {
                var labelName = vcardContact.displayLabel.label.split(" ");
                contact.firstName = labelName[0];
                if (labelName.length >1) {
                    //removes the first name
                    labelName.shift();
                    contact.lastName = labelName.toString().replace(",","");
                } else {
                    contact.lastName = "";
                }
            }
            contact.phone = vcardContact.phoneNumber.number;
            return contact;
        }

        onAddContactsAnswer: {
            telegramClient.contactsImportContactsAnswer.disconnect(contactParser.addContactsAnswer);
            busy = false;

            console.log("Imported " + contactIds.length + " contacts out of " + importedContactCount);
            dialogTitle = contactIds.length > 0 ? i18n.tr("Contacts imported") : i18n.tr("No contacts imported");
            dialogText = i18n.tr("This contact is on Telegram.",
                                 "%1 out of %2 contacts are on Telegram.",
                                 importedContactCount).arg(contactIds.length).arg(importedContactCount);
            PopupUtils.open(contactImportDialogComponent);
        }

        onVcardParsed: {
            if (contacts.length === 0) {
                busy = false;
                return;
            }
            console.log("Parsed " + contacts.length + " contacts.");
            if (contacts.length === 1) {
                var singleContact = parseContact(contacts[0]);
                showAddContactPage(true, -1, singleContact);
                busy = false;
            } else {
                var contactList = [];
                for (var i = 0; i < contacts.length; i++) {
                    var contact = parseContact(contacts[i]);
                    if (contact.phone !== "" && contact.firstName !== "") {
                        contactList.push(contact);
                    }
                }
                telegramClient.contactsImportContactsAnswer.connect(contactParser.addContactsAnswer);
                importedContactCount = contacts.length;
                importTimer.importContacts(contactList);
            }
        }
    }

    Timer {
        property var contactList: []

        id: importTimer
        interval: 500;
        running: false;
        repeat: true;
        triggeredOnStart: false

        onTriggered: {
            if (isConnected) {
                telegramClient.contactsImportContacts(contactList, false);
                importTimer.stop();
            }
        }

        function importContacts(contactList) {
            importTimer.contactList = contactList;
            importTimer.start();
        }
    }

    Component {
         id: addContactComponent
         Dialog {
             id: addContactDialog
             title: i18n.tr("Add Contact")
             Button {
                 text: i18n.tr("From Address Book")
                 color: UbuntuColors.orange
                 onClicked: {
                     PopupUtils.close(addContactDialog)
                     contactImporter.requestContact();
                 }
             }
             Button {
                 text: i18n.tr("Enter Details")
                 color: UbuntuColors.orange
                 onClicked: {
                     PopupUtils.close(addContactDialog)
                     showAddContactPage(true);
                 }
             }
             Button {
                 text: i18n.tr("Cancel")
                 onClicked: PopupUtils.close(addContactDialog)
             }
         }
    }

    Component {
        id: addContactToGroupChatComponent
        Dialog {
            property int userId: -1

            id: addContactToGroupChat
            title: i18n.tr("Telegram")
            text: i18n.tr("Text")
            TextField {
                id: messagesToForward
                anchors {
                    left: parent.left
                    leftMargin: units.gu(1)
                    right: parent.right
                    rightMargin: units.gu(1)
                }
                inputMethodHints: Qt.ImhDigitsOnly
                horizontalAlignment: Text.AlignHCenter
                text: "50"
            }

            Button {
                text: i18n.tr("OK")
                color: UbuntuColors.orange
                onClicked: {
                    var msgCount = parseInt(messagesToForward.text) || 20;
                    if (msgCount < 0) msgCount = 0;
                    addContactToGroupChatTimer.startWithData(userId, msgCount);
                    PopupUtils.close(addContactToGroupChat);
                }
            }
            Button {
                text: i18n.tr("Cancel")
                onClicked: PopupUtils.close(addContactToGroupChat)
            }

            function setData(userId, userName, groupName) {
                addContactToGroupChat.userId = userId;
                var text = i18n.tr("Add %1 to %2?\n\nNumber of last messages to forward:")
                        .arg(userName).arg(groupName);
                addContactToGroupChat.text = text;
            }
        }
    }

    Timer {
        property int userId: -1
        property int forwardedMessageCount: 20

        id: addContactToGroupChatTimer
        interval: 300
        repeat: false

        onTriggered: {
            telegramClient.addChatUser(groupChatId, userId, forwardedMessageCount);
            pageStack.pop();
        }

        function startWithData(userId, msgCount) {
            addContactToGroupChatTimer.userId = userId;
            addContactToGroupChatTimer.forwardedMessageCount = msgCount;
            start();
        }
    }

    onGroupChatModeChanged: {
        if (groupChatMode) {
            contactListView.startSelection();
            // onSelectedItemsChanged not working? So, call this:
        } else {
            contactListView.cancelSelection();
        }
        contactListView.refreshSubtitle();
    }

    function addPressed() {
        PopupUtils.open(addContactComponent);
    }

    function createChatPressed() {
        Qt.inputMethod.commit();

        var isSimpleChat = contactListView.isInSelectionMode ?
                    contactListView.selectedItems.count == 1 : true;

        var isGroupChat = !isSimpleChat && contactListView.selectedItems.count > 1;
        var hasGroupTitle = groupChatTitleTextField.text.length > 0;

        if (isSimpleChat || (isGroupChat && hasGroupTitle)) {
            contactListView.endSelection();
        }
    }

    function onBackPressed() {
        contactListView.cancelSelection();
    }

    function cancelChatPressed() {
        contactListView.cancelSelection();
    }

    function createChat(items) {
        if (items.count === 1) {
            var contact = items.get(0).model;
            openSimpleChat(contact);
        } else {
            openGroupChat(items);
        }
    }

    function openSimpleChat(contact) {
        var title = contact.firstName + " " + contact.lastName;
        searchFinished();
        pageStack.pop();
        openChat(title, contact.id, TLInputPeer.TypeInputPeerContact);
    }

    function openGroupChat(contacts) {
        var chatContacts = [];
        for (var i = 0; i < contacts.count; i++) {
            var contact = contacts.get(i).model;

            var inputUser = Qt.createQmlObject(
                        'import Telegram 0.1; TLInputUser {}', contactListView);
            inputUser.userId = contact.id;
            inputUser.classType = TLInputUser.TypeInputUserContact;

            chatContacts.push(inputUser);
        }

        telegramClient.messagesCreateChat(chatContacts, groupChatTitleTextField.text);
        searchFinished();
        pageStack.pop();
    }
}
