import QtQuick 2.0
import Ubuntu.Components 0.1
import "../components"

TelegramPage {
    property alias codeTextField: codeTextField
    property alias errorLabel: errorLabel
    property alias countdownText: countdownLabel.text
    property alias countdownTimer: countdownTimer

    id: page
    title: i18n.tr("Your code")
    onlineIndicationOnly: true

    function onBackPressed() {
        stopTimer();
    }

    body: Item {
        anchors {
            fill: parent
            margins: units.gu(2)
        }

        TelegramLabel {
            id: infoLabel
            anchors {
                top: parent.top
                margins: units.gu(1)
            }
            width: parent.width
            text: i18n.tr("We've sent an SMS with an activation code.\nPlease enter it below.\n")
        }

        TelegramLabel {
            id: countdownLabel
            anchors {
                top: infoLabel.bottom
                topMargin: units.gu(1)
            }
            width: parent.width
            // TRANSLATORS: the argument refers to a countdown time
            text: i18n.tr("We will call you in %1").arg(countdownTimer.getTimeAsText())
        }

        TelegramLabel {
            id: errorLabel
            anchors {
                top: countdownLabel.bottom
                topMargin: units.gu(1)
            }
            width: parent.width
            visible: false
            color: "red"
        }

        TgTextField {
            id: codeTextField
            anchors {
                top: errorLabel.bottom
                topMargin: units.gu(1)
            }
            inputMethodHints: Qt.ImhDigitsOnly

            validator: RegExpValidator {
                regExp: /[\w]+/
            }
            placeholderText: i18n.tr("Code")

            Keys.onEnterPressed: done()
            Keys.onReturnPressed: done()

            Component.onCompleted: {
                forceActiveFocus();
            }
        }

        TelegramButton {
            id: doneButton
            anchors {
                top: codeTextField.bottom
                topMargin: units.gu(1)
                right: parent.right
                left: parent.left
            }
            width: parent.width

            enabled: isConnected && codeTextField.text !== ""
            text: i18n.tr("Enter")
            onClicked: done()
        }
    }

    Timer {
        id: countdownTimer
        property int timeToCall: 120
        property int seconds: timeToCall
        interval: 1000
        repeat: true
        running: false

        onTriggered: {
            countdownLabel.text = i18n.tr("We will call you in %1").arg(getTimeAsText());
            seconds--;
            if (seconds < 0) {
                stop();
                telegramClient.authSendCall();
            }
        }

        Component.onCompleted: startTimer()

        function getTimeAsText() {
            var min = Math.floor(seconds / 60);
            var sec = (seconds % 60).toString()

            var pad = "00";
            sec = pad.substring(0, pad.length - sec.length) + sec;
            return min + ':' + sec;
        }
    }

    signal error(int id, int errorCode, int errorText);
    signal authSignInError();
    signal authLoggedIn();

    onError: {
        if (errorCode === 420) {
            setError(i18n.tr("Please wait a moment and try again"));
        } else if (errorCode === 400) {
            // handled in onAuthSignInError
        } else {
            console.log("VerificationCode error: " + errorCode + " " + errorText);
        }
        busy = false;
    }

    onAuthSignInError: {
        setError(i18n.tr("Incorrect code. Please try again."));
        busy = false;
    }

    Component.onCompleted: {
        telegramClient.error.connect(error);
        telegramClient.authSignInError.connect(authSignInError);
        telegramClient.authLoggedIn.connect(authLoggedIn);
    }

    Component.onDestruction: {
        telegramClient.authLoggedIn.disconnect(authLoggedIn);
        telegramClient.authSignInError.disconnect(authSignInError);
        telegramClient.error.disconnect(error);
    }

    function startTimer() {
        countdownTimer.seconds = countdownTimer.timeToCall;
        countdownTimer.start();
    }

    function stopTimer() {
        countdownTimer.stop();
    }

    function calling() {
        countdownLabel.text = i18n.tr("Calling you...");
    }

    function done() {
        if (busy) return;

        Qt.inputMethod.commit();
        Qt.inputMethod.hide();

        if (codeTextField.text.length > 0) {
            busy = true;
            clearError();
            if (userData.firstName.length > 0) {
                telegramClient.authSignUp(codeTextField.text,
                        userData.firstName, userData.lastName);
            } else {
                telegramClient.authSignIn(codeTextField.text);
            }
        }
    }

    function onError(errorMessage) {
        codeTextField.text = "";
        busy = false;
        setError(errorMessage);
    }

    function setError(message) {
        errorLabel.text = message;
        errorLabel.visible = true;
    }

    function clearError() {
        if (errorLabel.visible) {
            errorLabel.visible = false;
            errorLabel.text = "";
        }
    }
}
